unit T_Newprop;

{$mode objfpc}{$H+}

interface

uses
 Classes, SysUtils,forms,typinfo,menus,controls,variants;

type

// Klasse zum Erweitern der Properties aller Oberflächen Komponenten ohne Eingriff ins Lazarus.
// Zum Erzeugen bitte die Procedure create_newprop_for(AOwner:tcomponent) benutzen.
// Diese prüft , ob ein Tnewprop schon in der Komponentenliste von Owner vorhanden ist.
// AOwner ist dann der Owner für newprop
// Beispiel
 tnewprop=class(tcontrol)
 private
  const fversion:string='V0.75 nichts Halbes und nichts Ganzes';  // nur mal zum Test , ob das funktioniert
 private
  FHtmllink:string;
  FTag1:integer;
  FTag2:integer;
  F_An_Aus:boolean;
  FKommazahl:Double;
  function getversion:string;
 public
  constructor Create(AOwner: TComponent); override;
 published
  // Die neu erfundenen Properties
  property htmllink:string read fhtmllink write fhtmllink;
  property tag1:integer read ftag1 write ftag1;
  property tag2:integer read ftag2 write ftag2;
  property an_aus:boolean read F_an_aus write F_an_aus;
  property kommazahl:double read FKommazahl write FKommazahl;
  property version:string read getversion;

  // verstecktes Prop sichtbar machen , Test , wenn man das mal brauchen sollte
  property Text;
  // usw......
end;

procedure patch_all_components;
procedure set_prop(c:tcomponent;propertyname:string;value:variant);
function get_prop(c:tcomponent;propertyname:string):variant;
procedure set_prop_from_string(s:string);
procedure create_newprop_for(AOwner:tcomponent);

implementation

// Wert eines Properties lesen
// Dies funktioniert für einige Basis-Properties , als auch für die neuen
// Properties z.B. der property htmllink,Tag1,Tag2, siehe oben bei tnewprop
// Beispiel :
// Var x:integer;
// Var s:string:
// Var vers:string;
// x:=strtoint(get_prop(form1,'tag1'));
// s:=get_prop(button1,'htmllink');
// vers:=get_prop(button1,'version'); // ließt die Versions-Konstante von tnewprop aus
function get_prop(c:tcomponent;propertyname:string):variant;

    function get_the_prop(AComponent:TComponent;propertyname:string):variant;
    var
      kind:TTypeKind;
    begin
     if (AComponent<>NIL)
      then
       begin
         // derzeit unterstützte Typen
         // Strings
         // Integer
         // Boolean
         kind:=PropType(AComponent,propertyname);
         if Kind in [tkString,tkLString,tkAString,tkWString]
          then result:=getStrProp(AComponent,propertyname);
         // Integer
         if Kind in [tkinteger,tkInt64]
          then result:=getOrdProp(AComponent,propertyname);
         // Boolean
         if Kind in [tkBool]
          then result:=getEnumProp(AComponent,propertyname);
         // Double
         if (Kind in [tkFloat])
          then result:=getFloatProp(AComponent,propertyname);

         // hier eventuell erweitern auf andere Typen
         // weitere Typen siehe unit unit typinfo , TTypeKind
       end;
    end;

begin
 result:='NULL'; // Soll bedeuten , das es das Property nicht gibt

 if (c=nil) then exit;

 // Nachschauen , ob es in der Basis-Class von c ein passendes Orginal-Property gibt
 if IsPublishedProp(c,propertyname)
  then
   begin
    result:=get_the_prop(c,propertyname);
   end
  else
   // wenn nein , nachschauen ,ob es die Komponente Newprop gibt und hier dann
   // das gesuchte Property suchen
   begin
    c:=c.FindComponent('newprop');
    if (c<>nil) and (IsPublishedProp(c,propertyname))
     then
      begin
       result:=get_the_prop(c,propertyname);
      end
   end;
end;

// Wert eines Properties schreiben
// Dies funktioniert für einige Basis-Properties , als auch für die neuen
// Properties z.B. der property htmllink,Tag1,Tag2, siehe oben bei tnewprop
// Beispiel :
// set_prop(form1,'tag2',555);
// set_prop(button1,'htmllink','Button1.html');
procedure set_prop(c:TComponent;propertyname:string;value:variant);

     procedure set_the_prop(AComponent:TComponent;propertyname:string;value:variant);
     var
      kind:TTypeKind;
     begin
      if (AComponent<>NIL)
       then
        begin
          // derzeit unterstützte Typen
          // Strings
          // Integer
          // Int64
          // Boolean
          // Type des Propertys herausfinden
          kind:=PropType(AComponent,propertyname);

          // Strings
          if Kind in [tkString,tkLString,tkAString,tkWString]
           then
            SetStrProp(AComponent,propertyname,value);

          // Integer
          if (Kind in [tkinteger]) and
             (varType(value) =varinteger)
           then
            SetOrdProp(AComponent,propertyname,value);

          // Int64
          if (Kind in [tkInt64]) and
             (varType(value) = varint64)
           then
            SetOrdProp(AComponent,propertyname,value);

          // Boolean
          if (Kind in [tkBool]) and
             (varType(value) = varboolean)
           then
            SetEnumProp(AComponent,propertyname,value);

          // Double
          if (Kind in [tkFloat]) and
             (varType(value) = vardouble)
           then
            SetFloatProp(AComponent,propertyname,value);

          // hier eventuell erweitern auf andere Typen
          // weitere
          // Typen siehe unit unit typinfo , TTypeKind
          //             varianth.inc

        end;
     end;

begin
 if c=nil then exit;

 // Nachschauen , ob es in der Basis-Class von c ein passendes Orginal-Property gibt
 if IsPublishedProp(c,propertyname)
  then
   begin
    set_the_prop(c,propertyname,value);
   end
  else
   // wenn nein , nachschauen ,ob es die Komponente Newprop gibt und hier dann
   // das gesuchte Property suchen
   begin
    c:=c.FindComponent('newprop');
    if (c<>nil) and (IsPublishedProp(c,propertyname))
     then
      begin
       set_the_prop(c,propertyname,value);
      end
   end;
end;

// Die Version abfragen
// Ist nur ein Test , um zu versuchen , ob Konstanten in Klassen funktionieren
function tnewprop.getversion: string;
begin
 result:=fversion;
end;

constructor tnewprop.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 self.Name:='newprop';
 end;

// vorgelagerte Create procedure , um zu verhindern ,
// das newprop mehrfach angelegt wird ,
// sonst ballert es weil name schon vorhanden
procedure create_newprop_for(AOwner:tcomponent);
var c:tcomponent;
begin
 c:=AOwner.FindComponent('newprop');
 if c=nil
  then
   // wenn noch nicht vorhanden , dann anfügen
   tnewprop.create(AOwner);
end;

// Ein Property per String schreiben
//
// s:='Form1.button1.caption=Hallo Welt';
// set_prop_from_string(s);
//
// Hiermit könnte man dann z.B. alle Komponenten des Projektes aus einer
// Stringlist heraus umschreiben
// Interessant für Translation
procedure set_prop_from_string(s:string);
var var_wert:variant;
    // Hilfsvariablen
    bo_:boolean;
    int_:integer;
    flo_:double;
    sa:tstringarray;

    function GetFirstPart(var AString : String) : String;
     begin
      If (Pos('.', AString) > 0) then
       begin
        Result  := Copy(AString, 1, Pos('.', AString)-1);
        Delete(AString, 1, Pos('.', AString));
       end
      else
       begin
        Result  := AString;
        AString := '';
        end;
      AString   := Trim(AString);
      Result    := Trim(Result);
    end;

    function IsLastPart(AString : String) : Boolean;
    begin
      Result := (Pos('.', AString) = 0);
    end;

    // In Komponenten-Liste von AOwner nach eine Unterkomponente suchen
    function FindComponent(AOwner:TComponent;sName:String):TComponent;
    var i : Integer;
    begin
      Result := Nil;
      If (AOwner <> Nil) then
        With AOwner do
          For i:=0 to ComponentCount-1 do
            If (UpperCase(Components[i].Name) = UpperCase(sName)) then
              begin
               Result := Components[i];
               Break;
              end;

    end;

    // Komponente suchen und Eigenschaft ändern
    procedure Aendere_Eigenschaft(sProperty:string;wert:variant);
    var AComponent : TComponent;
        prop:string;
    begin
      sProperty    := Trim(sProperty);
      AComponent   := Application;

      If (sProperty <> '') then
        While (not IsLastPart(sProperty)) do
          AComponent := FindComponent(AComponent, GetFirstPart(sProperty))
      else
        exit;

      If ((AComponent <> Nil) and (vartype(wert) <> varEmpty)) then
        begin
         prop:=GetFirstPart(sProperty); // <--- Property von AComponent
         set_prop(AComponent,prop,wert);
        end;
    end;

begin
 // den Inputstring s z.B. 'Form1.button1.caption=Hallo Welt' ,
 // an '=' zerlegen
 sa:=s.Split('=');
 // in sa[0] steht dann 'Form1.button1.caption' , also Komponente und Property
 // in sa[1] steht dann der Werte , also 'Hallo Welt'

 // nun versuchen wir herauszufinden was das für ein Werte-Type ist
 // und weisen diesen Werte-Type der varinate (var_wert) zu.
 if trystrtoint(sa[1],int_) // auf integer testen
  then
   var_wert:=int_
  else
  if trystrtofloat(sa[1],flo_) // auf Float testen
   then
    var_wert:=flo_
   else
   if trystrtobool(sa[1],bo_) // auf bool testen
    then
     var_wert:=bo_
    else
     var_wert:=sa[1]; // wenn alles nicht passt , dann machen wir nen String draus

 // Weiter zu Eigenschaft ändern
 Aendere_Eigenschaft(sa[0],       // <--- Das Objekt mit Prpoerty
                     var_wert);   // der Wert als variant

end;



// Beispiel: siehe project1.lpr
// Alle Komponenten des Projektes durchjukeln und newprop anheften
// patch_all_components wird in der project1.lpr unterhalb der
//  Application.CreateForm eingetragen
(*
program project1;

{$mode objfpc}{$H+}

uses
 {$IFDEF UNIX}{$IFDEF UseCThreads}
 cthreads,
 {$ENDIF}{$ENDIF}
 Interfaces, // this includes the LCL widgetset
 Forms, Unit1,Unit1,

 T_Newprop
 { you can add units after this };

{$R *.res}

begin
 RequireDerivedFormResource:=True;
 Application.Scaled:=True;
 Application.Initialize;
 Application.CreateForm(TForm1, Form1);
 Application.CreateForm(TForm2, Form2);


 // Alle Komponenten mit newprop ausrüsten
 // Wichtig , erst müssen alle Forms erzeugt sein
 patch_all_components;

 Application.Run;
end.
*)
procedure patch_all_components;
var form_i:tform;
    comp:tcomponent;
    fi,ci:integer;
begin
 // alle Forms durchgehen
 for fi:=0 to screen.FormCount - 1 do
  begin
   // Aktuelles Form
   form_i:=forms.Screen.Forms[fi];
   create_newprop_for(form_i);// Schaut vorher nach , ob es schon ein Tnewprop in dem Form gibt
   // alle Komponenten des aktuellen Forms durchgehen
   for ci:=0 to form_i.ComponentCount-1 do
    begin
     // Eine Komponente aus der Komponentenliste des form_i
     comp:=form_i.Components[ci];
     create_newprop_for(comp); // Schaut vorher nach , ob es schon ein Tnewprop in der Komponente gibt
    end;
  end;
end;


end.

